import sqlalchemy as sa
from sqlalchemy.dialects.postgresql import UUID, VARCHAR, TIMESTAMP, SMALLINT, JSONB, TEXT
from sqlalchemy.ext.declarative import declarative_base
from uuid import uuid4

Base = declarative_base()

class Workers(Base):
    __tablename__ = "workers"

    uuid = sa.Column(UUID(as_uuid=True),primary_key=True,default=uuid4)
    persona = sa.Column(VARCHAR(12))
    name = sa.Column(VARCHAR(50))
    last_name = sa.Column(VARCHAR(50))
    email = sa.Column(VARCHAR(150))
    username = sa.Column(VARCHAR(30))
    status = sa.Column(SMALLINT)
    logged_at = sa.Column(TIMESTAMP)
    session_deadline = sa.Column(TIMESTAMP)
    created_at = sa.Column(TIMESTAMP,server_default=sa.func.current_timestamp())
    updated_at = sa.Column(TIMESTAMP,onupdate=sa.func.current_timestamp())
    session_id = sa.Column(VARCHAR(50))

class WorkerHistory(Base):
    __tablename__ = "worker_history"

    uuid = sa.Column(UUID(as_uuid=True),primary_key=True,default=uuid4)
    worker_uuid = sa.Column(UUID(as_uuid=True),sa.ForeignKey(Workers.uuid), nullable=False)
    status_code = sa.Column(VARCHAR(20))
    status = sa.Column(VARCHAR(20))
    reason = sa.Column(TEXT)
    created_at = sa.Column(TIMESTAMP,server_default=sa.func.current_timestamp())
    updated_at = sa.Column(TIMESTAMP,onupdate=sa.func.current_timestamp())

class Tasks(Base):
    __tablename__ = "tasks"

    uuid = sa.Column(UUID(as_uuid=True),primary_key=True,default=uuid4)
    worker_uuid = sa.Column(UUID(as_uuid=True),sa.ForeignKey(Workers.uuid), nullable=False)
    name = sa.Column(VARCHAR(150))
    data = sa.Column(JSONB)
    status_code = sa.Column(VARCHAR(5))
    status = sa.Column(VARCHAR(20))
    priority = sa.Column(SMALLINT)
    deadline = sa.Column(TIMESTAMP)
    requested_at = sa.Column(TIMESTAMP)
    created_at = sa.Column(TIMESTAMP,server_default=sa.func.current_timestamp())
    updated_at = sa.Column(TIMESTAMP,onupdate=sa.func.current_timestamp())

class TaskHistory(Base):
    __tablename__ = "task_history"

    uuid = sa.Column(UUID(as_uuid=True),primary_key=True,default=uuid4)
    task_uuid = sa.Column(UUID(as_uuid=True),sa.ForeignKey(Tasks.uuid), nullable=False)
    worker_uuid = sa.Column(UUID(as_uuid=True),sa.ForeignKey(Workers.uuid), nullable=False)
    data = sa.Column(JSONB)
    reason = sa.Column(TEXT)
    status_code = sa.Column(VARCHAR(5))
    status = sa.Column(VARCHAR(20))
    created_at = sa.Column(TIMESTAMP(timezone=True),server_default=sa.func.current_timestamp())
    updated_at = sa.Column(TIMESTAMP,onupdate=sa.func.current_timestamp())

if __name__ == "__main__":
    engine = sa.create_engine("postgresql://cloud:af9c7KHRugnCK7E@35.223.124.237:5432/task_manager_test")
    Base.metadata.create_all(bind=engine)